@@ -1,15 +1,20 @@
-# Copyright (c) 2013, The MITRE Corporation. All rights reserved.
+# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
 # See LICENSE.txt for complete terms.
 
 import datetime
 import unittest
 
 from cybox.common import (BaseProperty, DateTime, Integer, Long,
-        PositiveInteger, String, UnsignedLong, BINDING_CLASS_MAPPING)
+        NonNegativeInteger, PositiveInteger, String, UnsignedInteger,
+        UnsignedLong, BINDING_CLASS_MAPPING, DEFAULT_DELIM)
 import cybox.test
 from cybox.utils import normalize_to_xml
 
 
+NUMERIC_TYPES = (Integer, PositiveInteger, UnsignedInteger, NonNegativeInteger,
+                 Long, UnsignedLong)
+
+
 class TestBaseProperty(unittest.TestCase):
 
     def test_create_all(self):
@@ -35,6 +40,12 @@ class TestBaseProperty(unittest.TestCase):
         s2 = cybox.test.round_trip(s)
         self.assertEqual(s, s2)
 
+    def test_delimiter(self):
+        s = String(["string1", "string2"])
+        s.delimiter = "##delim##"
+        self.assertTrue("##comma##" not in s.to_xml())
+        self.assertTrue("string1##delim##string2" in s.to_xml())
+
     def test_integer(self):
         i = Integer(42)
         self.assertEqual(i.datatype, "integer")
@@ -156,6 +167,24 @@ class TestBaseProperty(unittest.TestCase):
         self.assertEqual(i.to_dict(), i2.to_dict())
 
 
+class TestEmptyNumerics(unittest.TestCase):
+
+    def test_empty_numeric_value(self):
+        # Integer, Long, UnsignedLong, PositiveInteger
+
+        for cls in NUMERIC_TYPES:
+            # Rather than importing from cybox.bindings directly, just
+            # reference the corresponding _binding_class.
+            binding_obj = cls._binding_class()
+
+            # Set value to an empty string
+            binding_obj.valueOf_ = ''
+
+            obj = cls.from_obj(binding_obj)
+
+            self.assertTrue(obj.value is None)
+
+
 class TestHexadecimal(unittest.TestCase):
 
     def test_parse_int(self):
@@ -203,10 +232,56 @@ class TestDateTime(unittest.TestCase):
     def test_list_dates(self):
         dt = DateTime([self.dt, self.dt, self.dt])
         self.assertEqual(3, len(dt.value))
-        expected = "{0},{0},{0}".format(self.dt.isoformat())
-        actual = normalize_to_xml(dt.serialized_value)
+        expected = "{0}{1}{0}{1}{0}".format(self.dt.isoformat(), DEFAULT_DELIM)
+        actual = normalize_to_xml(dt.serialized_value, DEFAULT_DELIM)
         self.assertEqual(expected, actual)
 
 
+class TestApplyCondition(unittest.TestCase):
+    """Make sure that the @apply_condition attribute is set at the right times.
+
+    If the @condition attribute is set, meaning it is part of a pattern, and
+    the value is a list, the @apply_condition should be set, even if it is
+    the default value ("ANY").
+    """
+
+    def test_instance_single(self):
+        s = String("foo")
+        # @apply_condition should not be set on instances.
+        self.assertFalse('apply_condition' in s.to_xml())
+
+    def test_instance_multiple(self):
+        s = String(["foo", "bar", "baz"])
+        # @apply_condition should not be set on instances.
+        self.assertFalse('apply_condition' in s.to_xml())
+
+    def test_instance_multiple_all(self):
+        s = String(["foo", "bar", "baz"])
+        s.apply_condition = "ALL"
+        # Even though we set it, this is not a pattern so @apply_condition
+        # shouldn't be output.
+        self.assertFalse('apply_condition' in s.to_xml())
+
+    def test_pattern_single(self):
+        s = String("foo")
+        s.condition = "Equals"
+        # @apply_condition should not be set if the value is not a list.
+        self.assertFalse('apply_condition' in s.to_xml())
+
+    def test_pattern_multiple(self):
+        s = String(["foo", "bar", "baz"])
+        s.condition = "Equals"
+        # @apply_condition should be set when there is a @condition and the
+        # value is a list.
+        self.assertTrue('apply_condition="ANY"' in s.to_xml())
+
+    def test_pattern_multiple_all(self):
+        s = String(["foo", "bar", "baz"])
+        s.condition = "Equals"
+        s.apply_condition = "ALL"
+        # If we change @apply_condition from the default, it should match
+        # that value.
+        self.assertTrue('apply_condition="ALL"' in s.to_xml())
+
 if __name__ == "__main__":
     unittest.main()
